#include "ball.h"

Ball::Ball(Vector cen, float rad, Color c, float accel, float _mass, float _maxSpeed)
: sphere(cen, rad, c),
	velocity(0,0,0),
	mass(_mass),
	acceleration(accel),
	rotVector(1,0,0),
	angSpeed(0),
	onGround(false),
	maxSpeed(_maxSpeed)
{
	// Initialize the stored rotation matrix
	glLoadIdentity();
	glGetFloatv(GL_MODELVIEW_MATRIX, currentRotation);
}

Ball::~Ball(void)
{
}

// Set the vector to rotate around when rolling, and the linear rolling speed of the ball.
void Ball::setRollingRotation(Vector _rotVector, float speed)
{
	rotVector = _rotVector;
	angSpeed = speed / (sphere.getRadius() * 2 * PI) * 360.0;
}
void Ball::setPosition(Vector newPosition)
{
	sphere.setCenter(newPosition);
}

// Advance the rolling rotation of the ball by timestep time units
void Ball::advanceRollingRotation(float timestep)
{
	glLoadIdentity();
	float rotAngle =  -angSpeed*timestep;
	if(rotAngle == rotAngle) // don't rotate if rotAngle is NAN (because NAN != NAN)
	{
		glRotatef(-angSpeed*timestep, rotVector.x, rotVector.y, rotVector.z);  
		glMultMatrixf(currentRotation);
		glGetFloatv(GL_MODELVIEW_MATRIX, currentRotation);
	}
}
