#pragma once
#include "Sphere.h"

class Ball
{
public:
	Ball(Vector cen, float rad, Color c, float accel, float _mass, float _maxSpeed);
	~Ball(void);

	void draw() const { sphere.drawRotated(currentRotation); }

	Vector getPosition() const { return sphere.getCenter(); }
	void setPosition(Vector newPosition);
	float getRadius() const { return sphere.getRadius(); }
	Vector getVelocity() const { return velocity; }
	void setVelocity(const Vector &newVelocity) { velocity = newVelocity; }
	void addVelocity(const Vector &delta) { setVelocity(velocity + delta); }
	void advanceRollingRotation(float timestep);
	void setRollingRotation(Vector _rotVector, float speed); // rotVector is vector to rotate around, speed is linear rolling speed
	bool canJump() { return onGround; } // can I jump?
	void setOnGround(bool newOnGround) { onGround = newOnGround; }
	float getMass() { return mass; }
	float getAcceleration() { return acceleration; }
	float getMaxSpeed() { return maxSpeed; }
	void setMass( float newMass ) { mass = newMass; }
	void setAcceleration( float newAccel ) { acceleration = newAccel; }
	void setMaxSpeed( float newMaxSpeed ) { maxSpeed = newMaxSpeed; }
private:
	Sphere sphere;
	float currentRotation[16];  // rotation matrix for current rotation
	Vector velocity;
	// units of velocity are "GL world units / timestep"
	Vector rotVector; // rolling rotation vector
	float angSpeed;   // angular speed in degrees/timestep
	static int numBalls;
	float mass;
	float acceleration;
	float maxSpeed;
	bool onGround; 
};
