#include "Breakable.h"
#include <stdlib.h>

const float Breakable::FALLING_SPEED = 0.1;
const float Breakable::HORIZONTAL_EXPLOSION_SPEED = 0.04;

Breakable::Breakable(char* filename, Vector offset)
	: Object(filename, offset, true), breaking(false), horizontalVelocities(NULL)
{
	minY = min(triangles[0]->getV1().y, min(triangles[0]->getV2().y, triangles[0]->getV3().y));
	for(int i = 1; i < numTriangles; ++i)
	{
		int thisMin = min(triangles[i]->getV1().y, min(triangles[i]->getV2().y, triangles[i]->getV3().y));
		if(thisMin < minY)
			minY = thisMin;
	}
}

Breakable::~Breakable()
{
}

void Breakable::collided(Ball& ball)
{
	if(!breaking)
	{
		breaking = true;
		collidable = false;

		horizontalVelocities = new Vector[numTriangles];
		for(int i = 0; i < numTriangles; ++i)
		{
			horizontalVelocities[i] = Vector(rand() - RAND_MAX/2, 0.0, rand() - RAND_MAX/2);
			horizontalVelocities[i].normalize();
			horizontalVelocities[i] = horizontalVelocities[i] * HORIZONTAL_EXPLOSION_SPEED;
		}
	}
}

void Breakable::advanceTime(float timestep)
{
	if(breaking)
	{
		bool done = true;
		for(int i = 0; i < numTriangles; ++i)
		{
			if(max(triangles[i]->getV1().y, max(triangles[i]->getV2().y, triangles[i]->getV3().y)) > minY)
			{
				done = false;
				triangles[i]->setPosition(triangles[i]->getV1() - Vector(0.0, FALLING_SPEED*timestep*50, 0.0) + horizontalVelocities[i]*timestep*50,
										  triangles[i]->getV2() - Vector(0.0, FALLING_SPEED*timestep*50, 0.0) + horizontalVelocities[i]*timestep*50,
										  triangles[i]->getV3() - Vector(0.0, FALLING_SPEED*timestep*50, 0.0) + horizontalVelocities[i]*timestep*50);
			}
		}
		if(done)
		{
			breaking = false;
			delete[] horizontalVelocities;
		}
	}
}
