#include "Object.h"
#include <iostream>
#include <fstream>

Object::Object(char *filename, Vector offset, bool _collidable)
	: collidable(_collidable), triangles(NULL), numTriangles(0)
{
	loadTriangles(filename, offset);

	// Find bounding sphere for the object as a whole:
	//   center is average of all its triangles' bounding sphere centers
	//   radius is distance from that center to furthest triangle's bounding sphere center
	//     plus that triangle's bounding sphere radius
    boundingSphereCenter = Vector(0.0, 0.0, 0.0);
	for(int i = 0; i < numTriangles; ++i)
		boundingSphereCenter = boundingSphereCenter + triangles[i]->getBoundingSphereCenter();
	boundingSphereCenter = boundingSphereCenter * (1.0 / numTriangles);

	boundingSphereRadius = 0;
	for(int i = 0; i < numTriangles; ++i)
	{
		float distance = (boundingSphereCenter - triangles[i]->getBoundingSphereCenter()).mag()
		   + triangles[i]->getBoundingSphereRadius();
		if(distance > boundingSphereRadius)
			boundingSphereRadius = distance;
	}
}

Object::~Object()
{
	for(int i = 0; i < numTriangles; ++i)
		delete triangles[i];
	delete[] triangles;
}

void Object::draw()
{
	for(int i = 0; i < numTriangles; ++i)
		triangles[i]->draw();
}

bool Object::loadTriangles(char* fileName, Vector offset)
{
	ifstream inFile(fileName, ios::in);  // open the input file

	if(!inFile.good())
	{
		cout << "Error: Couldn't open file " << fileName << endl;
		return false;                    // the filename wasn't valid
	}

	int numVertices;
	inFile >> numVertices;

	Vector* vertices = new Vector[numVertices];

	// Load in the list of vertices, offsetting them by the specified amount
	for(int i = 0; i < numVertices; i++)
	{
		inFile >> vertices[i].x >> vertices[i].y >> vertices[i].z;
		vertices[i] = vertices[i] + offset;
	}

	inFile >> numTriangles;
	triangles = new Triangle*[numTriangles];

	// Load in the list of triangles with colors
	for(int i = 0; i < numTriangles; i++)
	{
		int v1, v2, v3;
		float r, g, b;
		inFile >> v1 >> v2 >> v3 >> r >> g >> b;
		triangles[i] = new Triangle(vertices[v1], vertices[v2], vertices[v3], Color(r,g,b));
	}

	delete[] vertices;

	return true;   // file loaded successfully
}
