#pragma once

#include "includes.h"
#include "Triangle.h"
#include "Ball.h"

class Object
{
public:
	// Need to specify a file to load triangles from and an offset to load them at
	Object(char* filename, Vector offset, bool _collidable);

	~Object();

	// Draw all the object's triangles
	void draw();

	// Accessor: return a pointer to the array of triangle pointers
	Triangle** getTriangles() { return triangles; }
	// Accessor: return the number of triangles in the object
	int getNumTriangles() { return numTriangles; }
	// Accessors: get the bounding sphere info
	Vector getBoundingSphereCenter() { return boundingSphereCenter; }
	float getBoundingSphereRadius() { return boundingSphereRadius; }

	// This gets called so the object can do things, if necessary
	virtual void advanceTime(float timestep) { };

	// This gets called when the object is collided with, so it can react accordingly.
	// The colliding ball is passed.
	// (Note: actual collision response with the triangles is handled externally, this is
	//        just to do object-specific things, if any.)
	virtual void collided(Ball& ball) { };

	// If false, things don't bounce off this object's triangles.  Collisions are still
	// detected so that the collided() member function can be called to inform the object
	// of the collision.
	bool isCollidable() { return collidable; }

protected:
	Triangle** triangles;
	int numTriangles;

	// See explanation with the getCollidable() accessor above for what this does.
	bool collidable;

	Vector boundingSphereCenter;
	float boundingSphereRadius;

	// Given an object file, initializes this object using the triangles in that file,
	// offset by the given vector from the coordinates specified in the file.
	bool loadTriangles(char* fileName, Vector offset);
};