#include "Powerup.h"
#include <stdlib.h>

const float Powerup::MASS_INCREASE = 1.0;
const float Powerup::ACCELERATION_INCREASE = 3.0;
const float Powerup::MAX_SPEED_INCREASE = 10.0;

const float Powerup::EXPLOSION_DURATION = 80.0;
const float Powerup::EXPLOSION_SPEED = 0.07;

Powerup::Powerup(char* filename, Vector offset, Type _type)
	: Object(filename, offset, false), type(_type), timeSinceExplosion(0), exploding(false), explosionVelocities(NULL)
{
}

Powerup::~Powerup()
{
}

void Powerup::collided(Ball& ball)
{
	if(!exploding)
	{
		switch(type)
		{
		case MASS:
			ball.setMass(ball.getMass() + MASS_INCREASE);
			break;
		case ACCELERATION:
			ball.setAcceleration(ball.getAcceleration() + ACCELERATION_INCREASE);
			break;
		case MAX_SPEED:
			ball.setMaxSpeed(ball.getMaxSpeed() + MAX_SPEED_INCREASE);
			break;
		}

		exploding = true;
		explosionVelocities = new Vector[numTriangles];
		for(int i = 0; i < numTriangles; ++i)
		{
			explosionVelocities[i] = Vector(rand() - RAND_MAX/2, rand() - RAND_MAX/2, rand() - RAND_MAX/2);
			explosionVelocities[i].normalize();
			explosionVelocities[i] = explosionVelocities[i] * EXPLOSION_SPEED;
		}
	}
}

void Powerup::advanceTime(float timestep)
{
	if(exploding)
	{
		if(timeSinceExplosion > EXPLOSION_DURATION)
		{
			// make the powerup's triangles disappear
			for(int i = 0; i < numTriangles; ++i)
				delete triangles[i];
			numTriangles = 0;
			delete[] explosionVelocities;
			exploding = false;
		}
		else
		{
			for(int i = 0; i < numTriangles; ++i)
			{
				triangles[i]->setPosition(triangles[i]->getV1() + explosionVelocities[i],
									      triangles[i]->getV2() + explosionVelocities[i],
										  triangles[i]->getV3() + explosionVelocities[i]);

				timeSinceExplosion += timestep;
			}
		}
	}
}