#include "sphere.h"
#include <iostream>

using namespace std;

int Sphere::numSpheres = 0;

Sphere::Sphere(Vector cen, float rad, Color c)
: Shape(c), center(cen), radius(rad)
{
	numSpheres++;
	boundingSphereCenter = cen;
	boundingSphereRadius = rad;

	//  read in the bit map
	//  the file name is hard-coded
	BITMAPINFOHEADER	bitmapInfoHeader;	// bitmap info header
	bitmapData=LoadBitmapFile("ball3.bmp", &bitmapInfoHeader);

	// create the sphere
	theSphere= gluNewQuadric();
	gluQuadricTexture(theSphere, GL_TRUE);

	// create texture
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, hTexture);

	// the following two commands tell openGL how to interpolate texture
	// values between the sampled values in your texture bitmap
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

	// the following two commands tell openGL how to extend the texture
	// when needed
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

	// now give it the texture map
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, bitmapInfoHeader.biWidth,
				 bitmapInfoHeader.biHeight, 0, GL_RGB, GL_UNSIGNED_BYTE, bitmapData);

	// initialize the list
	hBall = glGenLists(1);
	glNewList(hBall, GL_COMPILE);

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, hTexture);

	int slices = 16;	// slices and stacks
	int stacks = 16;	// control the resolution

	glTranslatef(0.0f,0.0,0.0f);
	gluSphere(theSphere, 1.0, slices, stacks);
	glDisable(GL_TEXTURE_2D);

	// end the list
	glEndList();

}

Sphere::~Sphere(void)
{
	if(--numSpheres == 0)
		glDeleteTextures(1, (const unsigned int*) &hTexture);
	gluDeleteQuadric(theSphere);
	glDeleteLists(hBall,1);
}

void Sphere::draw() const
{
	// begin sphere draw
	glColor3f(color.r, color.g, color.b);
	glPushMatrix();
	glTranslatef(center.x, center.y, center.z); // move to center
	glScalef(radius, radius, radius);  //scale radius in x,y, and z
	glCallList(hBall);
	glPopMatrix(); //end sphere draw
}

void Sphere::drawRotated(const float* rotMatrix) const
{
	// begin sphere draw
	glColor3f(color.r, color.g, color.b);
	glPushMatrix();
	glTranslatef(center.x, center.y, center.z); // move to center
	glScalef(radius, radius, radius);  //scale radius in x,y, and z
	glMultMatrixf(rotMatrix);          // rotate the ball by the given matrix
	glCallList(hBall);
	glPopMatrix(); //end sphere draw
}