#define GLUT_DISABLE_ATEXIT_HACK
#include <iostream>
#include <Windows.h>
#include <GL/glut.h>
#include <fstream>
#include "world.h"
#include <math.h>
#include "includes.h"

int counter = 0;

using namespace std;

const int World::MAX_COMPUTERS = 100;
const float World::BALL_RADIUS = .3; // units: GL units
const float World::ELASTICITY = 0.60f;  // scaling factor applied to new velocity's perpendicular component after collision
const float World::FRICTION = 1;  // friction on each collision
const float World::GRAVITY = 6.5;    // units: (GL units) / timestep^2
const float World::ACCELERATION = 8; // Player acceleration per timestep
const float World::COMP_ACCELERATION = 6; // Computer ball default acceleration per timestep
const float World::MASS = 1.4;
const float World::COMP_MASS = 1.0;
const float World::B2B_ELASTICITY = 1.3; // Extra oomph per ball collision
const float World::MAX_ACCEL_SPEED = 20; // Initial speed past which players cannot accelerate; can change
  										 // (can still change direction, and can still
										 //  be accelerated by gravity and collisions).
const float World::JUMP_VEL = 5; // Jump velocity
const float World::YVEL_GROUND = .75; // a horrible name-- this is cutoff for when a collision will set the onGround attribute of a
									   // ball to TRUE. It is a velocity cutoff, not a position.
const float World::TURN_RATE = 3; // spped of turning, in some sort of units. 
const int World::BORDER = 50;   // number of pixels "border" -- zoom in/out to keep balls this
                                //  far from edge	of screen

World* World::pinstance = NULL;
World* World::instance()
{
	if(pinstance == NULL)
      pinstance = new World; // create the singleton World instance if not yet created

	return pinstance;
}

World::World(void)
: playerBall(Vector(0,0,0), BALL_RADIUS, Color(1.0, 1.0, 1.0), ACCELERATION, MASS, MAX_ACCEL_SPEED), theCamera(),
//numTriangles(0), triangles(NULL),
	objects(NULL), numObjects(0), mode(ARENA), numSlowSteps(0), power(0), numStrokes(0),
	moveFwd(false), moveBack(false), moveLeft(false), moveRight(false), moveJump(false), 
	turnRight(false), turnLeft(false), zoomFactor(0),
	computerBalls(new Ball*[MAX_COMPUTERS]), numComputerBalls(0),
	playerStart(0,0,0),
	AIStart(2,0,0)
{
	strcpy(keys, "qweasdc");
	playerBall.setVelocity(Vector(0.0, -0.01, 0.01));
	theCamera.setPosition(10, 30, 20);
	theCamera.lookAt(playerBall.getPosition());

//	for(int x = 0; x < MAX_COMPUTERS; ++x) 
//		computerBalls[x] = new Ball(playerStart+Vector(-(BALL_RADIUS+.5)*(x+1),0,0), BALL_RADIUS,
//								    Color(1.0, 0.0, ((float) x) / MAX_COMPUTERS), COMP_ACCELERATION, COMP_MASS);

	// We handle down/up keypresses, so don't want auto-repeats
	glutIgnoreKeyRepeat(true);
	// register glut event handlers
	glutMotionFunc(mouseMoveCallback);    
	glutMouseFunc(mousePressCallback);
	glutKeyboardFunc(keyPressCallback);
	glutKeyboardUpFunc(keyPressReleaseCallback);
	glutTimerFunc(100, physTimerCallback, 0);
	ticks1 = GetTickCount();
	startTicks = ticks1;
	lastSpawnTicks = startTicks;
	ticks2 = GetTickCount();
	DTicks = ticks2 - ticks1;
}

// Read level file and world triangles from file
bool World::loadFile(char* fileName)
{
//	cout << "Reading file " << fileName << endl;
	ifstream inFile(fileName, ios::in);  // open the input file

	if(!inFile.good())
	{
		cout << "Error: Couldn't open file " << fileName << endl;
		return false;                    // the filename wasn't valid
	}

	char buffer[1024];
	int numObjectsLoaded = 0;

	while(inFile >> buffer)
	{
		if(strcmp(buffer, "NUMOBJECTS") == 0)
		{
			inFile >> numObjects;
//			cout << "read NUMOBJECTS = " << numObjects << endl;
			objects = new Object*[numObjects];
		} else if(strcmp(buffer, "WORLDTRIANGLES") == 0)
		{
			inFile >> buffer;
//			cout << "read WORLDTRIANGLES = " << buffer << endl;
			objects[numObjectsLoaded++] = new Object(buffer, Vector(0.0, 0.0, 0.0), true);
		} else if(strcmp(buffer, "PLAYERSTART") == 0) 
		{
			float x,y,z;
			inFile >> x >> y >> z;
			playerStart = Vector(x,y,z);
//			cout << "read PLAYERSTART = " << playerStart << endl;
			playerBall.setPosition(playerStart);
		} else if(strcmp(buffer, "AISTART") == 0)
		{
			float x,y,z;
			inFile >> x >> y >> z;
			AIStart = Vector(x,y,z);
//			cout << "read AIStart = " << AIStart << endl;
		} else if(strcmp(buffer, "BREAKABLE") == 0)
		{
			inFile >> buffer; // filename for triangles/center point
//			cout << "read BREAKABLE = " << buffer << endl;
			float x,y,z;
			inFile >> x >> y >> z;
			objects[numObjectsLoaded++] = new Breakable(buffer, Vector(x, y, z));
		} else if(strcmp(buffer, "POWERUP") == 0)
		{
			int type;
			inFile >> type;
			inFile >> buffer; // filename for triangles/center point
//			cout << "read POWERUP = " << buffer << endl;
			float x,y,z;
			inFile >> x >> y >> z;
			objects[numObjectsLoaded++] = new Powerup(buffer, Vector(x, y, z), (Powerup::Type)type);

		} else if(strcmp(buffer, "AIBALL") == 0)
		{
			float x,y,z;
			inFile >> x >> y >> z;
			if(mode == SURVIVAL)
				continue;
//			cout << "read AIBALL = " << Vector(x,y,z) << endl;
			computerBalls[numComputerBalls++] = new Ball(Vector(x,y,z), BALL_RADIUS, Color(1, 0, 0), COMP_ACCELERATION, 
														 COMP_MASS, MAX_ACCEL_SPEED);
		} else if(strcmp(buffer, "GOALNEAR") == 0)
		{
			float x,y,z;
			inFile >> x >> y >> z;
			goalNear = Vector(x,y,z);
		} else if(strcmp(buffer, "GOALFAR") == 0)
		{
			float x,y,z;
			inFile >> x >> y >> z;
			goalFar = Vector(x,y,z);
		} else if(strcmp(buffer, "CAMERAPOS") == 0)
		{
			float r,t,p;
			inFile >> r >> t >> p;
			theCamera.setPosition(r, t, p);
		}else {
			cout << "unidentified input in level file: " << buffer <<  endl;
		}
	}

	inFile.close();
	return true;
}

World::~World(void)
{
	glutMotionFunc(NULL);     // unregister glut event handlers
	glutMouseFunc(NULL);
	glutKeyboardFunc(NULL);
	glutKeyboardUpFunc(NULL);
	for(int i = 0; i < numObjects; ++i)
		delete objects[i];
	delete[] objects;

	for(int i = 0; i < numComputerBalls; ++i)
		delete computerBalls[i];
	delete[] computerBalls;
}

void World::draw()
{
	// Initialize GL for a new drawing run
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glDisable(GL_TEXTURE_2D);
	glPolygonMode(GL_FRONT, GL_FILL);
	glPolygonMode(GL_BACK, GL_LINE);
	glEnable(GL_CULL_FACE);   // don't draw backs of triangles at all  
	// First we apply the camera transform to the current matrix. Then we draw everything.
	glLoadIdentity();

	theCamera.doTransform();

	// Do lighting
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_COLOR_MATERIAL);
	glColorMaterial ( GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE ) ;
	float diffuse[4] = {.9, .9, .9, 1};
	float specular[4] = { 0.0, 0.0, 0.0, 0.0 };
	float ambient[4] = {.2, .2, .2, 1};
	float emission[4] = {0, 0, 0, 1};
	float position[4] = {-1.0,  2.0, 1.0, 0.0};
	glLightfv(GL_LIGHT0, GL_AMBIENT, ambient );
	glLightfv(GL_LIGHT0, GL_SPECULAR, specular );
	glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
	glLightfv(GL_LIGHT0, GL_POSITION, position);
	glMaterialfv( GL_FRONT_AND_BACK, GL_EMISSION, emission);
	glMaterialfv( GL_FRONT_AND_BACK, GL_SPECULAR, specular);
	glShadeModel(GL_FLAT); 

	for(int i = 0; i < numObjects; ++i)
		objects[i]->draw();
	if(mode != DONE)
	{
		glShadeModel(GL_SMOOTH);
		playerBall.draw();

		for(int x = 0; x < numComputerBalls; ++x)
			computerBalls[x]->draw();

		glPolygonMode(GL_BACK, GL_FILL);
	}
	glutSwapBuffers();
}

bool World::advanceTimestep()
{
	counter = 0; 

	// if game is already over, just return
	if(mode == DONE)
		return false;

	theCamera.setRelativePosition(zoomFactor, 0, 0); // zoom the camera if zoomFactor is set

	// Player Loses if playerball is below -20 y units
	if((playerBall.getPosition().y) < -20.0)
	{
		if(mode == SURVIVAL)
		{	
			mode = DONE;
			char message[100];
			int time = (GetTickCount() - startTicks) / 1000;
			sprintf(message, "Game over.\nYou survived for %d seconds.", time);
			MessageBox(NULL, message, "Boooo!", MB_OK);
			exit(0);
		}
		mode = DONE;
		char message[100];
		sprintf(message, "You lose!\nHahaha, loser.");
		MessageBox(NULL, message, "Boooo!", MB_OK);
		exit(0);
		return false;
	}

	// Computer disappears if it falls below -10 y units
	for(int ball = 0; ball < numComputerBalls; ++ball)
	{
		if(computerBalls[ball]->getPosition().y < -10.0)
		{
			delete computerBalls[ball];

			for(int x = ball + 1; x < numComputerBalls; ++x)
				computerBalls[x-1] = computerBalls[x];

			--numComputerBalls;
			--ball;   // retry this index, since it's a new ball now
		}
	}

//	cout << mode << " " << numComputerBalls << endl;
	// Win conditions
	if(mode == ARENA && numComputerBalls == 0)
	{
		mode = DONE;
		char message[100];
		sprintf(message, "You win!");
		MessageBox(NULL, message, "So I did!", MB_OK);
		exit(0);
		return false;
	}
	// survival has no win conditions
	if(mode == SURVIVAL && (numComputerBalls == 0 || (GetTickCount() - lastSpawnTicks > 10000)))
	{
		float timeFactor = GetTickCount() - startTicks;
		timeFactor /= 10000;
		computerBalls[numComputerBalls++] = new Ball(AIStart, BALL_RADIUS + timeFactor/10, Color(1,0,0), COMP_ACCELERATION - timeFactor/10, COMP_MASS + timeFactor/2, MAX_ACCEL_SPEED + timeFactor);
		computerBalls[numComputerBalls - 1]->setVelocity(Vector(0,0,0));
		lastSpawnTicks = GetTickCount();
	}
	if(mode == RACE)
	{
		Vector pos = playerBall.getPosition();
		if( ( (pos.x > goalNear.x && pos.x < goalFar.x) || (pos.x < goalNear.x && pos.x > goalFar.x) )
			&& ( (pos.y > goalNear.y && pos.y < goalFar.y) || (pos.y < goalNear.y && pos.y > goalFar.y) )
			&& ( (pos.z > goalNear.z && pos.z < goalFar.z) || (pos.z < goalNear.z && pos.z > goalFar.z) ) )
			{
				mode = DONE;
				char message[100];
				int time = (GetTickCount() - startTicks) / 1000;
				sprintf(message, "You win!\nYou took %d seconds to finish the course.", time);
				MessageBox(NULL, message, "So I did!", MB_OK);
				exit(0);
				return false;
			}
	}

	// Handle a turning
	if (turnRight)
		theCamera.setRelativePosition(0, TURN_RATE, 0); 
	if (turnLeft)
		theCamera.setRelativePosition(0, -TURN_RATE, 0); 

	// Handle a jump if jumpkey is held down.
	if (moveJump)
	{
		if (playerBall.canJump()) {
			playerBall.addVelocity(Vector(0.0, JUMP_VEL, 0.0));
			playerBall.setOnGround(false);
		}
	}

	ticks1 = ticks2;
	ticks2 = GetTickCount();
	DTicks = ticks2 - ticks1;
	bool result = advanceTime((float)DTicks/1000.0);

	// center viewpoint on center of mass of the balls
	Vector centerMass = playerBall.getPosition();
	for(int ball = 0; ball < numComputerBalls; ++ball)
		centerMass = centerMass + computerBalls[ball]->getPosition();
	centerMass = centerMass * (1.0 / (1.0 + numComputerBalls));
	theCamera.setDesiredFocalPoint(centerMass);

	// zoom in/out to see all the balls, super smash bros. style

	// get modelview and projection matrices
	glPushMatrix();
	glLoadIdentity();
	theCamera.doTransform();
	float modelview[16];
	float projection[16];
	glGetFloatv(GL_MODELVIEW_MATRIX, modelview);
	glGetFloatv(GL_PROJECTION_MATRIX, projection);
	glPopMatrix();

	Vector playerOnScreen = ptOnScreen(modelview, projection, playerBall.getPosition());

	float maxOffScreen = max(abs(playerOnScreen.x), abs(playerOnScreen.y));

	for(int ball = 0; ball < numComputerBalls; ++ball)
	{
		Vector pos = computerBalls[ball]->getPosition();
		Vector ballOnScreen = ptOnScreen(modelview, projection, pos);
		float x = abs(ballOnScreen.x);
		float y = abs(ballOnScreen.y);
		maxOffScreen = max(maxOffScreen, max(x,y));
	}
	if(numComputerBalls > 0)
		theCamera.setMaxOffScreen(maxOffScreen);

	return result;
}

void World::collide(Ball& myBall, float timeStep) {

	counter++;

	if(myBall.getVelocity().magSq() == 0)
		return;

	Vector startPos = myBall.getPosition();
	Vector endPos = startPos + (myBall.getVelocity() * timeStep);

	Vector pathBoundingSphereCenter = (startPos + endPos) * 0.5;
	float pathBoundingSphereRadius = sqrt((endPos - pathBoundingSphereCenter).magSq())
									 + myBall.getRadius();

	float smallestAlpha = 1.01f;  // value in range [0,1] representing distance along path of the
								  // first collision (of any type)

	Vector collisionPt; // point of the actual collision (not the position of
						// center on the path at collision time; you can calculate
	                    // that from smallestAlpha)
						// NOTE: currently this is not used by face collisions, which have
						//       their own struct with a ->where attribute, or by ball-ball
						//       collisions, which don't need to know this information

	CollisionType collisionFound = NONE;

	// check for collision with another ball

	// make a vector with all the balls in it
	int numTotalBalls = numComputerBalls + 1;
	Ball** allBalls = new Ball*[numTotalBalls];
	allBalls[0] = &playerBall;
	for(int ball = 1; ball < numTotalBalls; ++ball)
		allBalls[ball] = computerBalls[ball - 1];
	Ball* collidedWithBall;

	// for each ball, check collision unless it's the same ball
	for(int ball = 0; ball < numTotalBalls; ++ball)
	{
		if(allBalls[ball] == &myBall)
			continue;

		// check if path bounding sphere intersects with other ball first
		if(checkSphereIntersection(pathBoundingSphereCenter, pathBoundingSphereRadius,
								   allBalls[ball]->getPosition(), allBalls[ball]->getRadius()))
		{
			// Find closest point to other ball on path
			float alpha = linePointMinimumDistance(startPos, endPos, allBalls[ball]->getPosition());

			// there's only a collision at all if we're <= R1 + R2 away
			float distance = sqrt((startPos + ((endPos - startPos) * alpha)
									- allBalls[ball]->getPosition()).magSq());

			Vector collisionDir = allBalls[ball]->getPosition() - myBall.getPosition();

			collisionDir.normalize();

			// make sure the balls are going toward each other
			if(myBall.getVelocity().dot(collisionDir) - allBalls[ball]->getVelocity().dot(collisionDir) < 0)
				continue;
//			if(myBall.getVelocity().dot(allBalls[ball]->getVelocity()) > 0 
//			   && myBall.getVelocity().magSq() <= allBalls[ball]->getVelocity().magSq())
//			   continue;
//			if(myBall.getVelocity().dot(collisionDir) < allBalls[ball]->getVelocity().dot(collisionDir))
//				continue;
//			if(collisionDir.dot(myBall.getVelocity() - allBalls[ball]->getVelocity()) <= 0.00001)
//				continue;

			if(distance <= myBall.getRadius() + allBalls[ball]->getRadius())
			{
//				if(alpha != 0 && alpha != 1)
//				{
//					float r = myBall.getRadius() + allBalls[ball]->getRadius();
//					float x = sqrt(r*r - distance * distance);
//					alpha -= x / (endPos - startPos).mag();
//				} //else {
					//// Back up to where we were R1 + R2 away (using interpolation)
					//float startDistance = sqrt((startPos - allBalls[ball]->getPosition()).magSq());
					//if(startDistance - distance == 0)
					//continue;
					//alpha *= 1.0 - (((myBall.getRadius() + allBalls[ball]->getRadius()) - distance)
					//			/ (startDistance - distance));
				//}
				

				// If this collision is the closest thus far, set it as the 'current' one
				if(alpha < smallestAlpha)
				{
					collisionFound = BALL;
					collidedWithBall = allBalls[ball];
					smallestAlpha = alpha;
				}
			}
		}
	}
	delete[] allBalls;

	// check for collisions with each object's triangles

	Collision* faceCollision = NULL;

	// For non-collidable objects, we still call their collided() function iff there are no
	// collidable-object collisions occuring earlier.  However, they don't count as "real" collisions,
	// so the first collidable-object collision occuring later is still handled as usual.

	float smallestNoncollidableAlpha = 1.01f;
	Object* nonCollidable = NULL;
	Object* collidedWith = NULL;

	for(int i = 0; i < numObjects; ++i)
	{
		// first do a bounding sphere check with the object as a whole
		if(checkSphereIntersection(pathBoundingSphereCenter, pathBoundingSphereRadius,
			objects[i]->getBoundingSphereCenter(), objects[i]->getBoundingSphereRadius()))
		{
			Triangle** triangles = objects[i]->getTriangles();
			int numTriangles = objects[i]->getNumTriangles();

			for(int x = 0; x < numTriangles; ++x)
			{
				// now do a bounding sphere check with this triangle
				if(checkSphereIntersection(pathBoundingSphereCenter, pathBoundingSphereRadius,
										triangles[x]->getBoundingSphereCenter(),
										triangles[x]->getBoundingSphereRadius()))
				{
					// Check if it collides with a face
					Collision* newFaceCollision = checkFaceCollision(startPos, endPos,
																	myBall.getRadius(), triangles[x]);
					if(newFaceCollision != NULL) // there was a face collision
					{
						// alpha is magnitude of projection of start->collisionPt onto path
						// divided by total length of path if there were no collisions
						float alpha = (newFaceCollision->where - startPos).dot((endPos-startPos).normalize())
											/ sqrt((endPos - startPos).magSq());
						if(objects[i]->isCollidable())
						{
							if(alpha < smallestAlpha)
							{
								collidedWith = objects[i];
								smallestAlpha = alpha;
								collisionFound = FACE;
								if(faceCollision != NULL)
									delete faceCollision;
					
								faceCollision = newFaceCollision;
							}
							else
								delete newFaceCollision;
						}
						else
						{
							if(alpha < smallestNoncollidableAlpha)
							{
								smallestNoncollidableAlpha = alpha;
								nonCollidable = objects[i];
							}
						}
					}
					else // if there wasn't a face collision, check for edge/corner collision
					{
						float alphas[6];
							
						alphas[0] = linePointMinimumDistance(startPos, endPos,
								  								triangles[x]->getV1());
						alphas[1] = linePointMinimumDistance(startPos, endPos,
																triangles[x]->getV2());
						alphas[2] = linePointMinimumDistance(startPos, endPos,
																triangles[x]->getV3());

						float betas[3] = { 0.0, 0.0, 0.0 };
						lineLineMinimumDistance(startPos, endPos,
												triangles[x]->getV1(), triangles[x]->getV2(),
												alphas[3], betas[0]);
						lineLineMinimumDistance(startPos, endPos,
												triangles[x]->getV2(), triangles[x]->getV3(),
												alphas[4], betas[1]);
						lineLineMinimumDistance(startPos, endPos,
												triangles[x]->getV3(), triangles[x]->getV1(),
												alphas[5], betas[2]);
						// find which of the collisions that actually happen (i.e. minimum distance
						// is <= ballRadius) happens earliest (i.e. has smallest alpha that's >= 0.0f),
						// and if any of them happen earlier than any already-existing edge or corner
						// collision from a previous triangle (i.e. have a smaller alpha than previousAlpha
						// and shorter distance than faceCollision)
						for(int v = 0; v < 6; v++)
						{
							Vector vertex;
							if(v == 0)
								vertex = triangles[x]->getV1();
							else if(v == 1)
								vertex = triangles[x]->getV2();
							else if(v == 2)
								vertex = triangles[x]->getV3();
							else if(v == 3)
								vertex = triangles[x]->getV1() + (triangles[x]->getV2() - triangles[x]->getV1()) * betas[0];
							else if(v == 4)
								vertex = triangles[x]->getV2() + (triangles[x]->getV3() - triangles[x]->getV2()) * betas[1];
							else
								vertex = triangles[x]->getV3() + (triangles[x]->getV1() - triangles[x]->getV3()) * betas[2];

							Vector closestPt = startPos + (endPos-startPos) * alphas[v];
							Vector collisionDir = vertex - closestPt;
							Vector vel = myBall.getVelocity();
							float closingVel = collisionDir.dot(vel); // closing velocity
							// if we're not moving toward the edge, don't collide
							if(closingVel <= 0)
								continue;

							// first check if the ball comes close enough
							if((closestPt - vertex).magSq() >= (myBall.getRadius() - 0.005) * (myBall.getRadius() - 0.005))
								continue;

							Vector pathCollisionPt;  // point on the path where we collide (i.e. where we're exactly
													 //  radius away from the collision point on the triangle

							if(alphas[v] < 1 && alphas[v] > 0)
							{
								float d = (vertex - closestPt).mag();
								float r = myBall.getRadius();
								float pathMag = (endPos - startPos).mag();
								float x = sqrt(r*r - d*d);
								alphas[v] = (x - alphas[v]*pathMag) / pathMag;
							}
							if(alphas[v] == 1)
							{
								float d = (vertex - closestPt).mag();
								float r = myBall.getRadius();
								float pathMag = (endPos - startPos).mag();
								alphas[v] -= (r - d) / pathMag;
							}// else if(alphas[v] == 0)
							 //	   continue;

							pathCollisionPt = startPos + (endPos - startPos) * alphas[v];

							if(objects[i]->isCollidable())
							{
								if(alphas[v] < smallestAlpha)
								{
									// if we get here, this is the closest collision so far
									collidedWith = objects[i];
									smallestAlpha = alphas[v];
									collisionPt = vertex;
									collisionFound = EDGE_OR_CORNER;
								}
							}
							else
							{
								if(alphas[v] < smallestNoncollidableAlpha)
								{
									nonCollidable = objects[i];
									smallestNoncollidableAlpha = alphas[v];
								}
							}
						}
					}
				}
			}
		}
	}

	// If there's a collision with a noncollidable object that comes before the first
	// response collision, call its notification function
	if(nonCollidable && smallestNoncollidableAlpha < smallestAlpha)
	{
		if(&myBall == &playerBall)
			nonCollidable->collided(myBall);
	}

	// If there's no collision, update the ball's position with straight-line motion
	if(collisionFound == NONE)
	{
		myBall.setPosition(endPos);
		myBall.advanceRollingRotation(timeStep);
	}
	else // otherwise handle the collision
	{
		// amount of timeStep remaining (will be calculated later, depending on collision type)
		float restOfTimeStep;
		
		// new velocity (also calculated later)
		Vector newVel;

		if(collisionFound == BALL)
		{
			Vector pathCollisionPt = startPos + (endPos - startPos) * smallestAlpha;
					 			 // point on the path where we collide (i.e. where we're exactly
			                     //  radius away from the collision point on the triangle
			restOfTimeStep = (1.0f - smallestAlpha) * timeStep;
			// move us to the collision
			Vector previousPosition = myBall.getPosition();
			myBall.setPosition(pathCollisionPt);

			// bounce the balls off each other
			// the code for different ball masses is here and works, just add the calls to find the correct masses
			float myBallMass = myBall.getMass();
			float collidedWithBallMass = collidedWithBall->getMass();

			Vector collDir = collidedWithBall->getPosition() - myBall.getPosition();
			collDir.normalize();
			
			// component of each ball's velocity in the collision direction
			float comp1 = collDir.dot(myBall.getVelocity());
			float comp2 = collDir.dot(collidedWithBall->getVelocity());
//			if(comp1 == 0 || comp2 == 0)
//				cout << "comp = 0" << endl;
			float totalMass = myBallMass + collidedWithBallMass;

//			// don't know why this is necessary
//			if(comp1 < 0)
//				comp1 = -comp1;
//			if(comp2 < 0)
//				comp2 = -comp2;

			float foo = comp1 - comp2;
//			if(foo < 0)
//				foo = -foo;
			foo *= 2;

			float distance = (previousPosition - collidedWithBall->getPosition()).mag();

			if(counter++ > 10)
			{
				return;
			}

			collidedWithBall->setVelocity(collidedWithBall->getVelocity() + collDir * (B2B_ELASTICITY*foo*(myBallMass/totalMass)));
			newVel = myBall.getVelocity() + collDir*(B2B_ELASTICITY*-foo*(collidedWithBallMass/totalMass));

		}
		else if(collisionFound == FACE)
		{
			// move us to the collision point
			myBall.setPosition(faceCollision->where);

			// To get the new ball velocity:  1. Transform the old velocity into the standard coordinate space, where the triangle
			// is horizontal.  2. Invert the y-component (vertical part) of the velocity.  3.  Transform the new velocity back into
			// real space.
			const Triangle* theTri = faceCollision->triangle;
			newVel = myBall.getVelocity();
			newVel = theTri->forwardTransform(newVel) - theTri->forwardTransform(Vector(0.0, 0.0, 0.0));
			newVel.y = -newVel.y * ELASTICITY;
			newVel = theTri->reverseTransform(newVel) - theTri->reverseTransform(Vector(0.0, 0.0, 0.0));

			// Friction - subtract (not multiply!) a small amount from the velocity
			Vector newVelNorm = newVel;
			newVelNorm.normalize();
			newVelNorm = newVelNorm * FRICTION * timeStep;
			if( newVel.magSq() > newVelNorm.magSq() )
				newVel = newVel - newVelNorm;
			else
				newVel = Vector(0,0,0); // friction brings us to a rolling stop

			// Ball rotation - update the rolling speed and direction
			Vector normal = theTri->reverseTransform(Vector(0.0, 1.0, 0.0)) - theTri->reverseTransform(Vector(0.0, 0.0, 0.0));
			float xSq = normal.dot(newVel);
			xSq *= xSq;
			float rollSpeed = sqrt(newVel.magSq() - xSq);
			Vector rotVector = newVel.cross(normal);
			if(!rotVector.magSq())  // if there is no vector to rotate around
				rollSpeed = 0;      // don't try to rotate

			restOfTimeStep = timeStep * (1.0 -
 					sqrt((faceCollision->where - startPos).magSq() / (endPos - startPos).magSq()));

			delete faceCollision;

			myBall.advanceRollingRotation(timeStep - restOfTimeStep); // roll the ball up to the collision
			myBall.setRollingRotation(rotVector, rollSpeed);

			// set whether it's on the ground
			float yVel = (myBall.getVelocity()).y;
			if (yVel<YVEL_GROUND && yVel>-YVEL_GROUND) // then we consider it to be rolling
				myBall.setOnGround(true);
			else
				myBall.setOnGround(false);

		}
		else // EDGE_OR_CORNER collision
		{
			Vector pathCollisionPt = startPos + (endPos - startPos) * smallestAlpha;
					 			 // point on the path where we collide (i.e. where we're exactly
			                     //  radius away from the collision point on the triangle
			restOfTimeStep = (1.0f - smallestAlpha) * timeStep;
			// move us to the collision
			myBall.setPosition(pathCollisionPt);

			Vector oldVel = myBall.getVelocity();
			Vector collisionDir = pathCollisionPt - collisionPt;
			collisionDir.normalize();
			float collMag = oldVel.dot(collisionDir);

			newVel = (collisionDir*collMag*-2*ELASTICITY) + oldVel;
			// Apply friction by subtracting (not multiplying)
			Vector newVelNorm = newVel;
			newVelNorm.normalize();
			newVelNorm = newVelNorm * FRICTION * timeStep;
			if( newVel.magSq() > newVelNorm.magSq() )
				newVel = newVel - newVelNorm;
			else
				newVel = Vector(0,0,0); // friction brings us to a rolling stop

			myBall.advanceRollingRotation(timeStep - restOfTimeStep); // roll up to the collision
			// update roll speed and direction
			Vector parallelVel = oldVel - (collisionDir * collMag);
			myBall.setRollingRotation(newVel.cross(collisionDir), sqrt(parallelVel.magSq()));
		}
		myBall.setVelocity(newVel);

		// if this was the player, let the object know it was collided with
		if(&myBall == &playerBall && collidedWith)
			collidedWith->collided(myBall);

		// Finish up the rest of the timestep
		collide(myBall, restOfTimeStep);
	}
}


bool World::advanceTime(float timeStep)
{
	theCamera.advanceTime(timeStep);
	for(int i = 0; i < numObjects; ++i)
		objects[i]->advanceTime(timeStep);

	// Basic AI routines. It would be better to have an AIBall class that has a function accelTo(vector).
	Vector playerAt = playerBall.getPosition();
	for(int ball = 0; ball < numComputerBalls; ++ball)
	{
		Vector ballAt = computerBalls[ball]->getPosition();
		Vector ballDir = computerBalls[ball]->getVelocity().normalize();
		Vector playerDir = playerAt - ballAt;
		playerDir.y = 0;
		Vector AIgoto = playerDir;
		AIgoto.y = 0;
		// lateral is a vector representing the sideways correction the ball should make to avoid orbiting and hit
		// the player head on
		Vector lateral = ballDir.perpTo(playerDir);
		lateral.y = 0;
		AIgoto = AIgoto - lateral * 10;
		AIgoto.y = 0;
		AIgoto.normalize();
		float oldSpeed = sqrt(computerBalls[ball]->getVelocity().magSq());
		float accel = computerBalls[ball]->getAcceleration();
		if(!computerBalls[ball]->canJump())
			accel /= 3.0;
		if(oldSpeed < computerBalls[ball]->getMaxSpeed())
			computerBalls[ball]->addVelocity(AIgoto * accel*timeStep);
		else if(computerBalls[ball]->getVelocity().dot(AIgoto) < 0)
			computerBalls[ball]->addVelocity(AIgoto * accel*timeStep);
		else
			computerBalls[ball]->addVelocity((AIgoto * accel).perpTo(computerBalls[ball]->getVelocity()*timeStep));
	}

	// Handle acceleration if any motion keys are currently held down
	if(moveFwd || moveBack || moveLeft || moveRight)
	{
		Vector moveDir(0.0, 0.0, 0.0);

		// theta (rotation in the x-z plane) of camera viewpoint determines the
		//  "forward" direction we're moving relative to (invert theta and result since
		//  the theta is from focal point to camera, and we want camera to focal point)
		Vector viewDir(-sin(-theCamera.getTheta() / 180.f * PI), 0.0,
					   -cos(-theCamera.getTheta() / 180.f * PI));

		if(moveFwd)
			moveDir = moveDir + viewDir;
		if(moveBack)
			moveDir = moveDir - viewDir;
		if(moveLeft)
			moveDir = moveDir + viewDir.cross(Vector(0.0, -1.0, 0.0));
		if(moveRight)
			moveDir = moveDir + viewDir.cross(Vector(0.0, 1.0, 0.0));

		moveDir.normalize();

		float oldSpeed = sqrt(playerBall.getVelocity().magSq());
		float accel = playerBall.getAcceleration();
		if(!playerBall.canJump())
			accel /= 3.0;
		if(oldSpeed < playerBall.getMaxSpeed())
		{
			playerBall.addVelocity(moveDir * accel*timeStep);
		} else {
			// the ball is going too fast, if the acceleration would cause the ball to go faster, ignore the component
			// in the direction of current movement
			if(playerBall.getVelocity().dot(moveDir) < 0)
				playerBall.addVelocity(moveDir * accel*timeStep);
			else
				playerBall.addVelocity((moveDir * accel).perpTo(playerBall.getVelocity()*timeStep));
		}
	}

	// apply some gravity
	playerBall.addVelocity(Vector(0.0, -GRAVITY*timeStep, 0.0));
	for(int ball = 0; ball < numComputerBalls; ++ball)
		computerBalls[ball]->addVelocity(Vector(0.0, -GRAVITY*timeStep, 0.0));

	collide(playerBall, timeStep);

 	for(int ball = 0; ball < numComputerBalls; ++ball)
		collide(*(computerBalls[ball]), timeStep);

	return (playerBall.getVelocity().magSq() < .00005);

}


void World::mouseMove(int newx, int newy)
{
	theCamera.setRelativePosition(0, newx - mousex, newy - mousey);
	mousex = newx;
	mousey = newy;
}

void World::mousePress(int button, int state, int x, int y)
{
	mousex = x;
	mousey = y;
}

void World::physTimer()
{
	glutTimerFunc(0, physTimerCallback, 0);
	if(mode == DONE)
		return;

	World::instance()->advanceTimestep();
	glutPostRedisplay();
}

void World::keyPress(unsigned char key, int x, int y)
{
	if(key == 'x')
		zoomFactor = 1.1;
	if(key == 'z')
		zoomFactor = 0.9;

	// Motion in x-z direction relative to camera direction (we always move within x-z plane
	//  so ignore y-component, which also nicely allows an overhead-looking-down camera
	//  view to work fine).

	// We actually just set booleans here indicating the user's depressed the key.  They get
	// cleared by keyPressRelease when the user releases the key, and their state is used
	// by advanceTimestep() to accelerate as desired.  This limits acceleration to once
	// per timestep, and avoids auto-repeat problems.
	if(key==keys[FORWARD])
		moveFwd = true;
	if(key==keys[BACKWARDS])
		moveBack = true;
	if(key==keys[STRAFELEFT])
		moveLeft = true;
	if(key==keys[STRAFERIGHT]) // strafe right
		moveRight = true;

	if(key==keys[TURNRIGHT])
		turnRight = true;
	if(key==keys[TURNLEFT])
		turnLeft = true;

	if(key == keys[JUMP] || key == ' ')
		moveJump = true;

	if(key=='f')
		exit(0);

	glutPostRedisplay();
}

void World::keyPressRelease(unsigned char key, int x, int y)
{
	// If user released one of the movement keys, update flag
	if(key == keys[FORWARD])
		moveFwd = false;
	if(key == keys[BACKWARDS])
		moveBack = false;
	if(key == keys[STRAFELEFT])
		moveLeft = false;
	if(key == keys[STRAFERIGHT])
		moveRight = false;
	if(key == keys[JUMP] || key == ' ')
		moveJump = false;

	if(key==keys[TURNRIGHT])
		turnRight = false;
	if(key==keys[TURNLEFT])
		turnLeft = false;

	if(key == 'z' || key == 'x')
		zoomFactor = 0;
}

void World::setKeys(char* _keys)
{
	strcpy(keys, _keys);
}
