// camera.cpp - implementation of camera class
#define GLUT_DISABLE_ATEXIT_HACK

#include <iostream>
#include <GL/glut.h>
#include "camera.h"

using namespace std;

Camera::Camera()
: focalPoint(Vector(0,0,0)), r(1), theta(0), phi(0), maxOffScreen(.6), desiredFocalPoint(0,0,0)
{
}

void Camera::lookAt(Vector newFocalPoint)
{
	focalPoint = newFocalPoint;
}

void Camera::doTransform() const
{
	glTranslatef(0, 0, -r);                                     // move r units back from focal point
	glRotatef(phi, 1, 0, 0);                                    // rotate phi degrees "up"
	glRotatef(theta, 0, 1, 0);                                  // rotate theta degrees "around"
	glTranslatef(-focalPoint.x, -focalPoint.y, -focalPoint.z);  // move to focal point
};

void Camera::setPosition(float newR, float newTheta, float newPhi)
{
	r = newR;
	theta = newTheta;
	phi = newPhi;
};

// move the camera like in super smash bros. to keep all balls in view
void Camera::advanceTime(float amount)
{
//	cout << "maxoffscreen " << maxOffScreen << endl;
	// zoom in/out slowly depending on the maximum distance of any ball from the center of
	// the screen, with 1 being the screen edge
	float scaleFactor = ((maxOffScreen + .4 - 1) / (2000 * amount)) + 1;
	if(scaleFactor < 10 && scaleFactor > 0)
		r *= scaleFactor;
	// move slowly to the current desired focal point (preventing sudden jumps)
	focalPoint = focalPoint + (desiredFocalPoint - focalPoint) * (amount * 2);
}

void Camera::setRelativePosition(float dr, float dTheta, float dPhi)
{
	if(dr != 0 &&  r*dr > .01 && r*dr < 90)
		r = r*dr;
	theta = theta + dTheta;
	phi = phi + dPhi;

	if(phi > 90)
		phi = 90;
	else if(phi < -90)
		phi = -90;

	if(r < 0)
		r = 0;
}

Camera::~Camera()
{
};
