// Collision-handling library functions.

#pragma once

#include "includes.h"
#include "Triangle.h"

// Different types of collisions that are possible
enum CollisionType { NONE, BALL, FACE, EDGE_OR_CORNER };

// Data structure returned by triangle face collisions
struct Collision
{
	Collision(Vector where_, const Triangle* tri) //Vector normal_)
		: where(where_), triangle(tri)      //normal(normal_)
	{}

	Vector where;
//	Vector normal;
	const Triangle* triangle;
};

// Quick check to see if two bounding spheres intersect.
bool checkSphereIntersection(Vector centerA, float radiusA,
 							 Vector centerB, float radiusB);

// Check collision with the face of a triangle, returning either the point of the
// collision (in a newly allocated Vector the caller must delete) or NULL if no collision
// Note: only checks for collisions with front faces!
Collision* checkFaceCollision(const Vector& pathStart, const Vector& pathEnd,
							  float ballRadius, const Triangle* triangle);

// Next two routines are used for detecting collisions with points and lines, respectively,
// and were mostly written by Prof. Z.  See comments in the .cpp file for details.
float linePointMinimumDistance(const Vector& start, const Vector& end, const Vector& v);
void lineLineMinimumDistance(const Vector& start1, const Vector& end1,
							 const Vector& start2, const Vector& end2,
							 float& alpha, float& beta);
