// Generic utility stuff
#include "includes.h"

float max(float a, float b)
{
	return a > b ? a : b;
}

float abs(float num)
{
	if(num < 0)
		num = -num;
	return num;
}

Vector transformPt(const float transform[16], const Vector& point)
{
	return Vector(transform[0 + 4*0] * point.x + transform[0 + 4*1] * point.y
		            + transform[0 + 4*2] * point.z + transform[0 + 4*3],
	  			  transform[1 + 4*0] * point.x + transform[1 + 4*1] * point.y
				    + transform[1 + 4*2] * point.z + transform[1 + 4*3],
				  transform[2 + 4*0] * point.x + transform[2 + 4*1] * point.y
				    + transform[2 + 4*2] * point.z + transform[2 + 4*3]);
}

Vector ptOnScreen(const float modelview[16], const float projection[16], const Vector& point)
{
	// multiply projection*modelview to get one transform matrix
	float transform[16];
	for(int col = 0; col < 4; ++col)
	{
		for(int row = 0; row < 4; ++row)
		{
			float val = 0;
			for(int src = 0; src < 4; ++src)
				val += projection[row + 4*src] * modelview[src + 4*col];
			transform[row + 4*col] = val;
		}
	}

	// transform, then scale by the 4th element of P*M*point where point is extended to
	// be a 4-element vector (with the 4th element 1); i.e. homogenous coordinates
	Vector result = transformPt(transform, point);
//	result.z = 0.0;
	float scale = transform[3 + 4*0] * point.x + transform[3 + 4*1] * point.y
						       + transform[3 + 4*2] * point.z + transform[3 + 4*3];
	if(scale == 0.0)
	{
		cerr << "why is scale 0? includes.cpp: ptOnScreen()" << endl;
		return Vector(0.0, 0.0, 0.0);
	}
	return result * (1.0 / scale);
}

ostream& operator<<(ostream& stream, Vector v)
{
	stream << v.x << " " << v.y << " " << v.z;
	return stream;
}

// LoadBitmapFile
// desc: Returns a pointer to the bitmap image of the bitmap specified
//       by filename. Also returns the bitmap header information.
//		 No support for 8-bit bitmaps.
unsigned char *LoadBitmapFile(char *filename, BITMAPINFOHEADER *bitmapInfoHeader)
{
	FILE *filePtr;							// the file pointer
	BITMAPFILEHEADER	bitmapFileHeader;		// bitmap file header
	unsigned char		*bitmapImage;			// bitmap image data
	unsigned int					imageIdx = 0;		// image index counter
	unsigned char		tempRGB;				// swap variable

	// open filename in "read binary" mode
	filePtr = fopen(filename, "r");
	if (filePtr == NULL)
		return NULL;

	// read the bitmap file header
	fread(&bitmapFileHeader, sizeof(BITMAPFILEHEADER), 1, filePtr);
	
	// verify that this is a bitmap by checking for the universal bitmap id
	if (bitmapFileHeader.bfType != BITMAP_ID)
	{
		fclose(filePtr);
		return NULL;
	}

	// read the bitmap information header
	fread(bitmapInfoHeader, sizeof(BITMAPINFOHEADER), 1, filePtr);

	// move file pointer to beginning of bitmap data
	fseek(filePtr, bitmapFileHeader.bfOffBits, SEEK_SET);

	// allocate enough memory for the bitmap image data
	bitmapImage = (unsigned char*)malloc(bitmapInfoHeader->biSizeImage);

	// verify memory allocation
	if (!bitmapImage)
	{
		free(bitmapImage);
		fclose(filePtr);
		return NULL;
	}

	// read in the bitmap image data
	fread(bitmapImage, 1, bitmapInfoHeader->biSizeImage, filePtr);

	// make sure bitmap image data was read
	if (bitmapImage == NULL)
	{
		fclose(filePtr);
		return NULL;
	}

	// swap the R and B values to get RGB since the bitmap color format is in BGR
	for (imageIdx = 0; imageIdx < bitmapInfoHeader->biSizeImage; imageIdx+=3)
	{
		tempRGB = bitmapImage[imageIdx];
		bitmapImage[imageIdx] = bitmapImage[imageIdx + 2];
		bitmapImage[imageIdx + 2] = tempRGB;
	}

	// close the file and return the bitmap image data
	fclose(filePtr);
	return bitmapImage;
}